;*******************************************************************************;
;   Microchip Technology Inc   2008
;	
;	This software shows why "swapf" must be used in INT restore procedure
;	To simulate interruption, a stimulus file is used containing just an asynchronous 
;   signal on RB0 (negative pulse). To simulate, fire RB0 in the Loop section in the first
;   part or second part.Eventually, the main will end up in error1 label showing that the
;   STATUS register has not been properly restored
;   
;   To change between Microchip (swapf) and movf, just change conditional assembly line
;   #define	MicrochipINT as a comment
;*******************************************************************************;

;=================================== HOUSEKEEPING

    IFDEF   __16F877                ; __<proc> is set by MPLAB-IDE when processor is selected
        #include p16F877.inc        ; include processor specific register definitions
    ENDIF
        
    IFDEF   __16F877A               ; __<proc> is set by MPLAB_IDE when processor is selected
        #include p16F877a.inc       ; include processor specific register definitions
    ENDIF
    
    IFDEF   __16F887                ; __<proc> is set by MPLAB_IDE when processor is selected
        #include p16F887.inc        ; include processor specific register definitions
    ENDIF  
    
    ;#define  MicrochipINT			; true = Microchip recommended INT handler with swapf
    								; false : alternative code using movf

	
;=================================== DATA MEMORY

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location

Push_Count  RES 1                   ; counter to hold number of time RB0 is pushed
Db_Ctr1     RES 1                   ; pair of counters used to create delay 
Db_Ctr2     RES 1                   ;  for debounce routine
;
w_temp      RES 1               ; variable used for context saving 
status_temp RES 1               ; variable used for context saving
pclath_temp RES 1               ; variable used for context saving

CommRAM     UDATA_SHR  0x071           ; Common RAM
Count		RES	1
RamZero		RES	1	
;=================================== PROGRAM MEMORY

;-----------------------------------
; VECTORS
;-----------------------------------
			
RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    
        
;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0
Main	
	clrf	Count
	                     
    bsf     STATUS,RP0              ; switch to bank 1   
    bsf     TRISB,0                 ; ### Initialize PORTB, bit 0 direction as input
    bsf     INTCON,INTE             ; ### Enable INT0 Interrupt
    bsf     INTCON,GIE              ; ### Enable Global Interrupt									 		
    bcf     STATUS,RP0              ; return to bank 0
    
    movlw	0						;;

Loop
	bsf		STATUS,Z
	nop
	nop
	nop
	nop
	btfss	STATUS,Z
	goto	error1
	bcf		STATUS,Z
	nop
	nop
	nop
	nop
	nop
	btfsc	STATUS,Z
	goto 	error0
    goto    Loop 
    
error0
	goto	$
error1
	goto	$

;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Nothing
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
#ifdef	MicrochipINT			; Microchip recommended backup method
	
    movwf   w_temp              ; save off current W register contents
    movf    STATUS,w            ; move status register into W register
    movwf   status_temp         ; save off contents of STATUS register
    movf    PCLATH,w            ; move pclath register into w register
    movwf   pclath_temp         ; save off contents of PCLATH register

#else							; proposed backup method
	movwf	w_temp
	movf 	STATUS,w 			; WREG = 0xFF (correct) et STATUS devient 0xFB
	movwf 	status_temp 		; valeur sauvegarde = 0xFF : correct

#endif
;
	incf	Count				; change bit 0

	btfss	Count,0				; to simulate change of STATUS bit
	goto	SetFlagZ
	movlw	1	
	goto	Common
SetFlagZ:
	movlw	0
Common:
	addwf	RamZero
	
    bcf     INTCON,INTF         ; clears the Interrupt request Flag
 ;
#ifdef	MicrochipINT			; Microchip recommended restore method
    movf    pclath_temp,w       ; retrieve copy of PCLATH register
    movwf   PCLATH              ; restore pre-isr PCLATH register contents
    movf    status_temp,w       ; retrieve copy of STATUS register
    movwf   STATUS              ; restore pre-isr STATUS register contents
    swapf   w_temp,f
    swapf   w_temp,w            ; restore pre-isr W register contents
 
 #else							; proposed restore method
	movf    status_temp,w       ;
	movwf	STATUS				; restore STATUS
 	movf 	w_temp				; restore W without swapf
 #endif
    retfie     
		
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
